"""
Python Library for communicating with the TI eZ430 Chronos watch via the
CC1111 USB RF Access Point.

Implemented :
Sync Date/Time, Calibrate Temp/Altitude
Get Button events
Get Accel Data
Get Sync Data
Send BlueRobin Data (Heartrate, Speed, Distance)

Not implemented :
RF BSL (Wireless Update) functions

TODO : 
Apparently TI released the code...grab constants from that...wish I knew
before starting this 2 hours ago...

Note :
I regret my function naming convention... I keep typing start_AP instead
of AP_start...  But easier to group with a prefix.
=====================
madox@madox.net
http://www.madox.net/
=====================

==========
Change Log
==========
2011-01-09
- Initial Version

============
Useful Links
============
TI Product Page :
http://focus.ti.com/docs/toolsw/folders/print/ez430-chronos.html

TI Wiki Page :
http://processors.wiki.ti.com/index.php/EZ430-Chronos

Google Groups :-
groups.google.com/group/ti-chronos-development-/
"""
__version__ = "$Revision: $"
# $Source$

import serial
import struct
import time
import binascii
        
class TIChronos:
  """
  TI Chronos Class
  """  
  def __init__(self, port="/dev/ttyACM0", debug=False):
    self.COM_open(port)
    self.BR_active = False
    self.AP_active = False
    self.sync_active = False
  """
  COM Port Functions
  """    
  def COM_open(self, port):
    #Open serial port with 115200 baud, 8,N,1 non-blocking mode,
    self.ser = serial.Serial(port, 115200, timeout=0)
  def COM_close(self):
    #Close the serial port
    self.ser.close()
  """
  HW Functions
  """      
  def HW_reset(self):
    #Reset Hardware
    self.send_command("\xFF\x01\x03")
  def HW_get_status(self):
    self.send_command("\xFF\x00\x04\x00")
    Data = self.read_data()
    if(len(Data)==4):
      """
      0 = Idle
      1 = AP Stopped
      2 = AP Attempting Link
      3 = AP Linked
      4 = BR Stopped
      5 = BR Transmitting
      """
      return ord(Data[3])
    else:
      return 0
  """
  Blue Robin Functions
  """    
  def BR_set_ID(self, id):
    #Stop BlueRobin ID
    #Little endian encoding for the ID
    Command = struct.pack('<3sI', "\xFF\x03\x07", id)
    self.send_command(Command)
    self.read_data()
  def BR_start(self):
    #Start BlueRobin TX
    self.send_command("\xFF\x02\x03")
    self.read_data()
    self.BR_active = True
  def BR_stop(self):
    #Stop BlueRobin TX
    self.send_command("\xFF\x06\x03")  
    self.BR_active = False
  def BR_set_heartrate(self, heartrate):
    #Set BlueRobin HeartRate
    Command = struct.pack('3sB', "\xFF\x05\x04", heartrate)
    self.send_command(Command)
  def BR_set_speeddistance(self, speed, distance):
    #Set BlueRobin Speed/Distance
    #Little endian encoding for the Distance
    Command = struct.pack('<3sBH', "\xFF\x0A\x06", speed, distance)
    self.send_command(Command)
    self.read_data()
  """
  AP Functions
  """  
  def AP_start(self):
    #Check if BlueRobin is active, if so stop it
    if(self.BR_active == True):
      self.BR_stop_TX()
      time.sleep(0.5) #Wait for stop to take effect
    self.send_command("\xFF\x07\x03")
    self.read_data()
    self.AP_active = True
  def AP_stop(self):
    #Send sync stop command
    self.sync_stop()
    self.send_command("\xFF\x09\x03")
    self.read_data()
    self.AP_active = False
  """
  sync Functions
  """    
  def sync_start(self):
    self.send_command("\xFF\x30\x03")
    self.read_data()
    self.sync_active = True
  def sync_send_command(self, command):
    #Send Sync Command, padded with 0x00 at end until 19 bytes
    Command = struct.pack('3s16s', "\xFF\x31\x16", command)
    self.send_command(Command)
    self.read_data()
  def sync_read_buffer(self):
    self.send_command("\xFF\x33\x03")
    Data = self.read_data()
    print binascii.b2a_hex(Data)
    print len(Data)
    print len(Data[3:22])
    return Data[3:22]
  def sync_get_buffer_status(self):
    self.send_command("\xFF\x32\x04\x00")
    Data = self.read_data()
    return Data[3]
  def sync_stop(self):
    self.sync_send_command("\x07")
    time.sleep(0.75)
    self.read_data()
    self.sync_active = False
  def sync_read_watch(self):
    if(self.AP_active == False):
      #TODO : Put a nice error message/exception here
      return
    if(self.sync_active == False):
      #TODO : Put a nice error message/exception here
      return
    #Request Watch Data
    self.sync_send_command("\x02")
    Data = ""
    for i in range(1,10):
      time.sleep(0.1)
      Status = ord(self.sync_get_buffer_status())
      if(Status==1):
        Data = self.sync_read_buffer()
        #Note Big Endian Encoding
        Status, Hours, Minutes, Seconds, Year, Month, Day, \
        AlarmHours, AlarmMinutes, \
        Temperature, Altitude, \
        LogMode, LogInterval, LogBytes = struct.unpack(">B3BH2B2B2H2BH", Data[0:18]) #Wonder what is in thebytes
        Hours = Hours & 0x7F            #Eliminate Bit 7 which is the 24 hour marker
        Temperature = float(Temperature) / 10  #Scale it back to Celcius
        WatchData = {"Status" : Status, "Hours" : Hours, "Minutes" : Minutes, "Seconds" : Seconds,\
                     "Year" : Year, "Month" : Month, "Day" : Day,\
                     "AlarmHours" : AlarmHours, "AlarmMinutes" : AlarmMinutes,\
                     "Temperature" : Temperature, "Altitude" : Altitude,\
                     "LogMode" : LogMode, "LogInterval" : LogInterval, "LogBytes" : LogBytes}
        return WatchData
    #TODO:Put a nice error/exception here
  def sync_write_watch(self, Hour, Minute, Second, Year, Month, Day, Temperature, Altitude):
    Hour = Hour & 0x80  #Set the 24 hour bit
    Temperature = Temperature * 10
    Command = struct.pack(">B3BH2B2B2H", 0x03, Hour, Minute, Second, Year, Month, Day, Temperature, Altitude)
    self.sync_send_command(Command)
    self.read_data()
  def sync_write_watch_now(self):
    CurrentDateTime = time.localtime()
    Hour = time.strftime("%H", CurrentDateTime) +128
    Minute = time.strftime("%M", CurrentDateTime) 
    Second = time.strftime("%S", CurrentDateTime) 
    Year = time.strftime("%Y", CurrentDateTime) - 1792
    Month = time.strftime("%m", CurrentDateTime) 
    Day = time.strftime("%d", CurrentDateTime) 
    self.sync_write_watch(Hour, Minute, Second, Year, Month, Day, 0, 0)
  """
  Data Functions
  """  
  def get_data(self):
    if(self.AP_active == False):
      #TODO : Put a nice error message/exception here
      return
    self.send_command("\xFF\x08\x07\x00\x00\x00\x00")
    Data=self.read_data()
    Header,Key,XX,YY,ZZ = struct.unpack("3sB3b", Data)
    if(Key == 0xFF):
      #Already read buffer
      return
    Button = ""
    Mode = ""
    if((Key & 0xF0) == 0x10):
      Button = "*"
      pass
    elif((Key & 0xF0) == 0x20):
      Button = "#"
      pass
    elif((Key & 0xF0) == 0x30):      
      Button = "Up"
      pass
    if(Key & 0x01):
      Mode = "Mouse"  #Mouse Mode (ACC)
      pass
    elif(Key & 0x02):
      Mode = "Button" #Button Mode (PPT)
      pass
    return {"Button" : Button, "Mode" : Mode, "AccelX" : XX, "AccelY" : YY, "AccelZ" : ZZ}
  def read_data(self):
    Data = []
    Input = self.ser.read()
    Data.append(Input)
    while(len(Input)!=0):
      Input = self.ser.read()
      Data.append(Input)
    return ''.join(Data)
  def send_command(self, data):
    #Write to serial port
    self.ser.write(data)
    #Add a 15ms delay after every command sent
    time.sleep(0.015)
